@@ -10,27 +10,26 @@ var LibraryBrowser = {
     '$setMainLoop',
     '$callUserCallback',
     '$safeSetTimeout',
+    '$warnOnce',
     'emscripten_set_main_loop_timing',
-#if !MINIMAL_RUNTIME
-    '$runtimeKeepalivePush',
-    '$runtimeKeepalivePop'
-#endif
   ],
-  $Browser__postset: 'Module["requestFullscreen"] = function Module_requestFullscreen(lockPointer, resizeCanvas) { Browser.requestFullscreen(lockPointer, resizeCanvas) };\n' + // exports
+  $Browser__postset: `
+    // exports
+    Module["requestFullscreen"] = function Module_requestFullscreen(lockPointer, resizeCanvas) { Browser.requestFullscreen(lockPointer, resizeCanvas) };
 #if ASSERTIONS
-                     'Module["requestFullScreen"] = function Module_requestFullScreen() { Browser.requestFullScreen() };\n' +
+    Module["requestFullScreen"] = function Module_requestFullScreen() { Browser.requestFullScreen() };
 #endif
-                     'Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };\n' +
-                     'Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };\n' +
-                     'Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };\n' +
-                     'Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };\n' +
-                     'Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() };\n' +
-                     'Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) };\n' +
+    Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
+    Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
+    Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
+    Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
+    Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() };
+    Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) };
 #if MAIN_MODULE
-                     'var preloadedWasm = {};\n' +
+    var preloadedWasm = {};
 #endif
-                     'var preloadedImages = {};\n' +
-                     'var preloadedAudios = {};\n',
+    var preloadedImages = {};
+    var preloadedAudios = {};`,
 
   $Browser: {
     mainLoop: {
@@ -112,12 +111,12 @@ var LibraryBrowser = {
         Browser.hasBlobConstructor = true;
       } catch(e) {
         Browser.hasBlobConstructor = false;
-        out("warning: no blob constructor, cannot create blobs with mimetypes");
+        err("warning: no blob constructor, cannot create blobs with mimetypes");
       }
-      Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? out("warning: no BlobBuilder") : null));
+      Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? err("warning: no BlobBuilder") : null));
       Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
       if (!Module.noImageDecoding && typeof Browser.URLObject == 'undefined') {
-        out("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
+        err("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
         Module.noImageDecoding = true;
       }
 
@@ -204,10 +203,10 @@ var LibraryBrowser = {
           assert(typeof url == 'string', 'createObjectURL must return a url as a string');
 #endif
           var audio = new Audio();
-          audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
+          audio.addEventListener('canplaythrough', () => finish(audio), false); // use addEventListener due to chromium bug 124926
           audio.onerror = function audio_onerror(event) {
             if (done) return;
-            out('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
+            err('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
             function encode64(data) {
               var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
               var PAD = '=';
@@ -259,14 +258,12 @@ var LibraryBrowser = {
           // than just running the promises in parallel, this makes a chain of
           // promises to run in series.
           wasmPlugin['asyncWasmLoadPromise'] = wasmPlugin['asyncWasmLoadPromise'].then(
-            function() {
-              return loadWebAssemblyModule(byteArray, {loadAsync: true, nodelete: true});
-            }).then(
-              function(module) {
+            () => loadWebAssemblyModule(byteArray, {loadAsync: true, nodelete: true})).then(
+              (module) => {
                 preloadedWasm[name] = module;
                 onload();
               },
-              function(err) {
+              (err) => {
                 console.warn("Couldn't instantiate wasm: " + name + " '" + err + "'");
                 onerror();
               });
@@ -292,12 +289,12 @@ var LibraryBrowser = {
                                     canvas['mozRequestPointerLock'] ||
                                     canvas['webkitRequestPointerLock'] ||
                                     canvas['msRequestPointerLock'] ||
-                                    function(){};
+                                    (() => {});
         canvas.exitPointerLock = document['exitPointerLock'] ||
                                  document['mozExitPointerLock'] ||
                                  document['webkitExitPointerLock'] ||
                                  document['msExitPointerLock'] ||
-                                 function(){}; // no-op if function does not exist
+                                 (() => {}); // no-op if function does not exist
         canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
 
         document.addEventListener('pointerlockchange', pointerLockChange, false);
@@ -306,7 +303,7 @@ var LibraryBrowser = {
         document.addEventListener('mspointerlockchange', pointerLockChange, false);
 
         if (Module['elementPointerLock']) {
-          canvas.addEventListener("click", function(ev) {
+          canvas.addEventListener("click", (ev) => {
             if (!Browser.pointerLock && Module['canvas'].requestPointerLock) {
               Module['canvas'].requestPointerLock();
               ev.preventDefault();
@@ -443,8 +440,8 @@ var LibraryBrowser = {
       canvasContainer.requestFullscreen = canvasContainer['requestFullscreen'] ||
                                           canvasContainer['mozRequestFullScreen'] ||
                                           canvasContainer['msRequestFullscreen'] ||
-                                         (canvasContainer['webkitRequestFullscreen'] ? function() { canvasContainer['webkitRequestFullscreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null) ||
-                                         (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
+                                         (canvasContainer['webkitRequestFullscreen'] ? () => canvasContainer['webkitRequestFullscreen'](Element['ALLOW_KEYBOARD_INPUT']) : null) ||
+                                         (canvasContainer['webkitRequestFullScreen'] ? () => canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) : null);
 
       canvasContainer.requestFullscreen();
     },
@@ -713,7 +710,7 @@ var LibraryBrowser = {
     setFullscreenCanvasSize: function() {
       // check if SDL is available
       if (typeof SDL != "undefined") {
-        var flags = {{{ makeGetValue('SDL.screen', '0', 'i32', 0, 1) }}};
+        var flags = {{{ makeGetValue('SDL.screen', '0', 'u32') }}};
         flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
         {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}};
       }
@@ -724,7 +721,7 @@ var LibraryBrowser = {
     setWindowedCanvasSize: function() {
       // check if SDL is available
       if (typeof SDL != "undefined") {
-        var flags = {{{ makeGetValue('SDL.screen', '0', 'i32', 0, 1) }}};
+        var flags = {{{ makeGetValue('SDL.screen', '0', 'u32') }}};
         flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
         {{{ makeSetValue('SDL.screen', '0', 'flags', 'i32') }}};
       }
@@ -779,11 +776,7 @@ var LibraryBrowser = {
     },
   },
 
-  emscripten_run_preload_plugins__deps: ['$PATH',
-#if !MINIMAL_RUNTIME
-    '$runtimeKeepalivePush', '$runtimeKeepalivePop',
-#endif
-  ],
+  emscripten_run_preload_plugins__deps: ['$PATH'],
   emscripten_run_preload_plugins__proxy: 'sync',
   emscripten_run_preload_plugins__sig: 'iiii',
   emscripten_run_preload_plugins: function(file, onload, onerror) {
@@ -796,11 +789,11 @@ var LibraryBrowser = {
       PATH.dirname(_file),
       PATH.basename(_file),
       new Uint8Array(data.object.contents), true, true,
-      function() {
+      () => {
         {{{ runtimeKeepalivePop() }}}
         if (onload) {{{ makeDynCall('vi', 'onload') }}}(file);
       },
-      function() {
+      () => {
         {{{ runtimeKeepalivePop() }}}
         if (onerror) {{{ makeDynCall('vi', 'onerror') }}}(file);
       },
@@ -809,9 +802,6 @@ var LibraryBrowser = {
     return 0;
   },
 
-#if !MINIMAL_RUNTIME
-  emscripten_run_preload_plugins_data__deps: ['$runtimeKeepalivePush', '$runtimeKeepalivePop'],
-#endif
   emscripten_run_preload_plugins_data__proxy: 'sync',
   emscripten_run_preload_plugins_data__sig: 'viiiiii',
   emscripten_run_preload_plugins_data: function(data, size, suffix, arg, onload, onerror) {
@@ -828,11 +818,11 @@ var LibraryBrowser = {
       name,
       {{{ makeHEAPView('U8', 'data', 'data + size') }}},
       true, true,
-      function() {
+      () => {
         {{{ runtimeKeepalivePop() }}}
         if (onload) {{{ makeDynCall('vii', 'onload') }}}(arg, cname);
       },
-      function() {
+      () => {
         {{{ runtimeKeepalivePop() }}}
         if (onerror) {{{ makeDynCall('vi', 'onerror') }}}(arg);
       },
@@ -874,7 +864,7 @@ var LibraryBrowser = {
         }
       }
     };
-    script.onerror = function() {
+    script.onerror = () => {
       {{{ runtimeKeepalivePop() }}}
       if (onerror) onerror();
     };
@@ -922,14 +912,15 @@ var LibraryBrowser = {
         // Emulate setImmediate. (note: not a complete polyfill, we don't emulate clearImmediate() to keep code size to minimum, since not needed)
         var setImmediates = [];
         var emscriptenMainLoopMessageId = 'setimmediate';
-        var Browser_setImmediate_messageHandler = function(/** @type {Event} */ event) {
+        /** @param {Event} event */
+        var Browser_setImmediate_messageHandler = (event) => {
           // When called in current thread or Worker, the main loop ID is structured slightly different to accommodate for --proxy-to-worker runtime listening to Worker events,
           // so check for both cases.
           if (event.data === emscriptenMainLoopMessageId || event.data.target === emscriptenMainLoopMessageId) {
             event.stopPropagation();
             setImmediates.shift()();
           }
-        }
+        };
         addEventListener("message", Browser_setImmediate_messageHandler, true);
         setImmediate = /** @type{function(function(): ?, ...?): number} */(function Browser_emulated_setImmediate(func) {
           setImmediates.push(func);
@@ -962,7 +953,6 @@ var LibraryBrowser = {
     'emscripten_webgl_commit_frame',
 #endif
 #if !MINIMAL_RUNTIME
-    '$runtimeKeepalivePush',
     '$maybeExit',
 #endif
   ],
@@ -1114,7 +1104,7 @@ var LibraryBrowser = {
   emscripten_set_main_loop_arg__deps: ['$setMainLoop'],
   emscripten_set_main_loop_arg__sig: 'viiii',
   emscripten_set_main_loop_arg: function(func, arg, fps, simulateInfiniteLoop) {
-    var browserIterationFunc = function() { {{{ makeDynCall('vi', 'func') }}}(arg); };
+    var browserIterationFunc = () => {{{ makeDynCall('vi', 'func') }}}(arg);
     setMainLoop(browserIterationFunc, fps, simulateInfiniteLoop, arg);
   },
 
@@ -1162,11 +1152,11 @@ var LibraryBrowser = {
   },
 
   // Runs natively in pthread, no __proxy needed.
-  emscripten_async_call__sig: 'viii',
+  emscripten_async_call__sig: 'vppi',
   emscripten_async_call__deps: ['$safeSetTimeout'],
   emscripten_async_call: function(func, arg, millis) {
     function wrapper() {
-      {{{ makeDynCall('vi', 'func') }}}(arg);
+      {{{ makeDynCall('vp', 'func') }}}(arg);
     }
 
     if (millis >= 0
@@ -1292,9 +1282,6 @@ var LibraryBrowser = {
 
   emscripten_call_worker__proxy: 'sync',
   emscripten_call_worker__sig: 'viiiiii',
-#if !MINIMAL_RUNTIME
-  emscripten_call_worker__deps: ['$runtimeKeepalivePush'],
-#endif
   emscripten_call_worker: function(id, funcName, data, size, callback, arg) {
     funcName = UTF8ToString(funcName);
     var info = Browser.workers[id];
