@@ -336,7 +336,7 @@ var LibraryExceptions = {
   // unwinding using 'if' blocks around each function, so the remaining
   // functionality boils down to picking a suitable 'catch' block.
   // We'll do that here, instead, to keep things simpler.
-  __cxa_find_matching_catch__deps: ['$exceptionLast', '$ExceptionInfo', '__resumeException', '__cxa_can_catch'],
+  __cxa_find_matching_catch__deps: ['$exceptionLast', '$ExceptionInfo', '__resumeException', '__cxa_can_catch', 'setTempRet0'],
   //__cxa_find_matching_catch__sig: 'p',
   __cxa_find_matching_catch: function() {
     var thrown = exceptionLast;
@@ -353,7 +353,6 @@ var LibraryExceptions = {
       setTempRet0(0);
       return thrown;
     }
-    var typeArray = Array.prototype.slice.call(arguments);
 
     // can_catch receives a **, add indirection
 #if EXCEPTION_DEBUG
@@ -363,8 +362,8 @@ var LibraryExceptions = {
     // Due to inheritance, those types may not precisely match the
     // type of the thrown object. Find one which matches, and
     // return the type of the catch block which should be called.
-    for (var i = 0; i < typeArray.length; i++) {
-      var caughtType = typeArray[i];
+    for (var i = 0; i < arguments.length; i++) {
+      var caughtType = arguments[i];
       if (caughtType === 0 || caughtType === thrownType) {
         // Catch all clause matched or exactly the same type is caught
         break;
