@@ -752,7 +752,7 @@ var splitModuleProxyHandler = {
 #if LOAD_SOURCE_MAP
 function receiveSourceMapJSON(sourceMap) {
   wasmSourceMap = new WasmSourceMap(sourceMap);
-  {{{ runOnMainThread("removeRunDependency('source-map');") }}}
+  {{{ runIfMainThread("removeRunDependency('source-map');") }}}
 }
 #endif
 
@@ -771,7 +771,6 @@ function instantiateSync(file, info) {
       // to load ok, but we do actually recompile the binary every time).
       var cachedCodeFile = '{{{ WASM_BINARY_FILE }}}.' + v8.cachedDataVersionTag() + '.cached';
       cachedCodeFile = locateFile(cachedCodeFile);
-      requireNodeFS();
       var hasCached = fs.existsSync(cachedCodeFile);
       if (hasCached) {
 #if RUNTIME_LOGGING
@@ -966,10 +965,10 @@ function createWasm() {
 
   }
   // we can't run yet (except in a pthread, where we have a custom sync instantiator)
-  {{{ runOnMainThread("addRunDependency('wasm-instantiate');") }}}
+  {{{ runIfMainThread("addRunDependency('wasm-instantiate');") }}}
 
 #if LOAD_SOURCE_MAP
-  {{{ runOnMainThread("addRunDependency('source-map');") }}}
+  {{{ runIfMainThread("addRunDependency('source-map');") }}}
 #endif
 
   // Prefer streaming instantiation if available.
@@ -1143,7 +1142,12 @@ function createWasm() {
       return exports;
     } catch(e) {
       err('Module.instantiateWasm callback failed with error: ' + e);
-      return false;
+      #if MODULARIZE
+        // If instantiation fails, reject the module ready promise.
+        readyPromiseReject(e);
+      #else
+        return false;
+      #endif
     }
   }
 
