@@ -428,10 +428,6 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       self.skipTest('no dynamic linking with memory growth (without wasm)')
     if not self.is_wasm():
       self.skipTest('no dynamic linking support in wasm2js yet')
-    if '-fsanitize=address' in self.emcc_args:
-      self.skipTest('no dynamic linking support in ASan yet')
-    if '-fsanitize=leak' in self.emcc_args:
-      self.skipTest('no dynamic linking support in LSan yet')
     if '-fsanitize=undefined' in self.emcc_args:
       self.skipTest('no dynamic linking support in UBSan yet')
 
@@ -491,10 +487,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     super().setUp()
     self.settings_mods = {}
     self.emcc_args = ['-Werror', '-Wno-limited-postlink-optimizations']
-    # We want to be strict about closure warnings in our test code.
-    # TODO(sbc): Remove this if we make it the default for `-Werror`:
-    # https://github.com/emscripten-core/emscripten/issues/16205):
-    self.ldflags = ['-sCLOSURE_WARNINGS=error']
+    self.ldflags = []
     self.node_args = [
       # Increate stack trace limit to maximise usefulness of test failure reports
       '--stack-trace-limit=50',
@@ -611,15 +604,18 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     return os.path.join(self.get_dir(), *pathelems)
 
   def add_pre_run(self, code):
-    create_file('prerun.js', 'Module.preRun = function() { %s }' % code)
+    assert not self.get_setting('MINIMAL_RUNTIME')
+    create_file('prerun.js', 'Module.preRun = function() { %s }\n' % code)
     self.emcc_args += ['--pre-js', 'prerun.js']
 
   def add_post_run(self, code):
-    create_file('postrun.js', 'Module.postRun = function() { %s }' % code)
+    assert not self.get_setting('MINIMAL_RUNTIME')
+    create_file('postrun.js', 'Module.postRun = function() { %s }\n' % code)
     self.emcc_args += ['--pre-js', 'postrun.js']
 
   def add_on_exit(self, code):
-    create_file('onexit.js', 'Module.onExit = function() { %s }' % code)
+    assert not self.get_setting('MINIMAL_RUNTIME')
+    create_file('onexit.js', 'Module.onExit = function() { %s }\n' % code)
     self.emcc_args += ['--pre-js', 'onexit.js']
 
   # returns the full list of arguments to pass to emcc
@@ -676,6 +672,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       compiler.append('-sNO_DEFAULT_TO_CXX')
 
     if force_c:
+      assert shared.suffix(filename) != '.c', 'force_c is not needed for source files ending in .c'
       compiler.append('-xc')
 
     if output_basename:
@@ -688,11 +685,8 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       cmd += emcc_args
     if libraries:
       cmd += libraries
-    if shared.suffix(filename) not in ('.i', '.ii'):
-      # Add the location of the test file to include path.
-      cmd += ['-I.']
-      if includes:
-        cmd += ['-I' + str(include) for include in includes]
+    if includes:
+      cmd += ['-I' + str(include) for include in includes]
 
     self.run_process(cmd, stderr=self.stderr_redirect if not DEBUG else None)
     self.assertExists(output)
@@ -756,6 +750,37 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     non_data_lines = [line for line in wat_lines if '(data ' not in line]
     return len(non_data_lines)
 
+  def clean_js_output(self, output):
+    """Cleaup the JS output prior to running verification steps on it.
+
+    Due to minification, when we get a crash report from JS it can sometimes
+    contains the entire program in the output (since the entire program is
+    on a single line).  In this case we can sometimes get false positives
+    when checking for strings in the output.  To avoid these false positives
+    and the make the output easier to read in such cases we attempt to remove
+    such lines from the JS output.
+    """
+    lines = output.splitlines()
+    long_lines = []
+
+    def cleanup(line):
+      if len(line) > 2048:
+        # Sanity check that this is really the emscripten program/module on
+        # a single line.
+        assert line.startswith('var Module=typeof Module!="undefined"')
+        long_lines.append(line)
+        line = '<REPLACED ENTIRE PROGRAM ON SINGLE LINE>'
+      return line
+
+    lines = [cleanup(l) for l in lines]
+    if not long_lines:
+      # No long lines found just return the unmodified output
+      return output
+
+    # Sanity check that we only a single long line.
+    assert len(long_lines) == 1
+    return '\n'.join(lines)
+
   def run_js(self, filename, engine=None, args=None,
              output_nicerizer=None,
              assert_returncode=0,
@@ -800,18 +825,20 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
       ret += read_file(stderr_file)
     if output_nicerizer:
       ret = output_nicerizer(ret)
+    if assert_returncode != 0:
+      ret = self.clean_js_output(ret)
     if error or timeout_error or EMTEST_VERBOSE:
-      ret = limit_size(ret)
       print('-- begin program output --')
-      print(read_file(stdout_file), end='')
+      print(limit_size(read_file(stdout_file)), end='')
       print('-- end program output --')
       if not interleaved_output:
         print('-- begin program stderr --')
-        print(read_file(stderr_file), end='')
+        print(limit_size(read_file(stderr_file)), end='')
         print('-- end program stderr --')
     if timeout_error:
       raise timeout_error
     if error:
+      ret = limit_size(ret)
       if assert_returncode == NON_ZERO:
         self.fail('JS subprocess unexpectedly succeeded (%s):  Output:\n%s' % (error.cmd, ret))
       else:
@@ -945,7 +972,9 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     # get_library() is used to compile libraries, and not link executables,
     # so we don't want to pass linker flags here (emscripten warns if you
     # try to pass linker settings when compiling).
-    emcc_args = self.get_emcc_args(ldflags=False)
+    emcc_args = []
+    if not native:
+      emcc_args = self.get_emcc_args(ldflags=False)
 
     hash_input = (str(emcc_args) + ' $ ' + str(env_init)).encode('utf-8')
     cache_name = name + ','.join([opt for opt in emcc_args if len(opt) < 7]) + '_' + hashlib.md5(hash_input).hexdigest() + cache_name_extra
@@ -1001,7 +1030,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
 
   # Shared test code between main suite and others
 
-  def expect_fail(self, cmd, **args):
+  def expect_fail(self, cmd, expect_traceback=False, **args):
     """Run a subprocess and assert that it returns non-zero.
 
     Return the stderr of the subprocess.
@@ -1011,7 +1040,9 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
     # When we check for failure we expect a user-visible error, not a traceback.
     # However, on windows a python traceback can happen randomly sometimes,
     # due to "Access is denied" https://github.com/emscripten-core/emscripten/issues/718
-    if not WINDOWS or 'Access is denied' not in proc.stderr:
+    if expect_traceback:
+      self.assertContained('Traceback', proc.stderr)
+    elif not WINDOWS or 'Access is denied' not in proc.stderr:
       self.assertNotContained('Traceback', proc.stderr)
     return proc.stderr
 
@@ -1224,13 +1255,15 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
                               interleaved_output=interleaved_output)
       js_output = js_output.replace('\r\n', '\n')
       if expected_output:
+        if type(expected_output) not in [list, tuple]:
+          expected_output = [expected_output]
         try:
           if assert_identical:
             self.assertIdentical(expected_output, js_output)
           elif assert_all or len(expected_output) == 1:
             for o in expected_output:
               if regex:
-                self.assertTrue(re.search(o, js_output), 'Expected regex "%s" to match on:\n%s' % (regex, js_output))
+                self.assertTrue(re.search(o, js_output), 'Expected regex "%s" to match on:\n%s' % (o, js_output))
               else:
                 self.assertContained(o, js_output)
           else:
@@ -1559,7 +1592,7 @@ class BrowserCore(RunnerCore):
             if extra_tries > 0:
               print('[test error (see below), automatically retrying]')
               print(e)
-              return self.run_browser(html_file, message, expected, timeout, extra_tries - 1)
+              return self.run_browser(html_file, expected, message, timeout, extra_tries - 1)
             else:
               raise e
       finally:
