@@ -4,7 +4,6 @@
 # found in the LICENSE file.
 
 import os
-import shutil
 import logging
 
 TAG = '2b147ffef10ec541d3eace326eafe11a54e635f8'
@@ -24,13 +23,8 @@ def get(ports, settings, shared):
 
   def create(final):
     logging.info('building port: sdl2_gfx')
-
     source_path = os.path.join(ports.get_dir(), 'sdl2_gfx', 'sdl2_gfx-' + TAG)
-    dest_path = ports.clear_project_build('sdl2_gfx')
-
-    shutil.copytree(source_path, dest_path)
-    ports.build_port(dest_path, final, [dest_path], exclude_dirs=['test'], flags=['-sUSE_SDL=2'])
-
+    ports.build_port(source_path, final, 'sdl2_gfx', exclude_dirs=['test'], flags=['-sUSE_SDL=2'])
     ports.install_headers(source_path, target='SDL2')
 
   return [shared.Cache.get_lib('libSDL2_gfx.a', create)]
