@@ -28,6 +28,14 @@ LIBC_SOCKETS = ['socket.c', 'socketpair.c', 'shutdown.c', 'bind.c', 'connect.c',
                 'getsockopt.c', 'setsockopt.c', 'freeaddrinfo.c',
                 'in6addr_any.c', 'in6addr_loopback.c', 'accept4.c']
 
+# Experimental: Setting EMCC_USE_NINJA will cause system libraries to get built with ninja rather
+# than simple subprocesses.  The primary benefit here is that we get accurate dependency tracking.
+# This means we can avoid completely rebuilding a library and just rebuild based on what changed.
+#
+# Setting EMCC_USE_NINJA=2 means that ninja will automatically be run for each library needed at
+# link time.
+USE_NINJA = int(os.environ.get('EMCC_USE_NINJA', '0'))
+
 
 def files_in_path(path, filenames):
   srcdir = utils.path_from_root(path)
@@ -49,7 +57,7 @@ def get_base_cflags(force_object_files=False):
   if settings.RELOCATABLE:
     flags += ['-sRELOCATABLE']
   if settings.MEMORY64:
-    flags += ['-sMEMORY64=' + str(settings.MEMORY64)]
+    flags += ['-Wno-experimental', '-sMEMORY64=' + str(settings.MEMORY64)]
   return flags
 
 
@@ -75,6 +83,7 @@ def run_build_commands(commands):
   # to setup the sysroot itself.
   ensure_sysroot()
   shared.run_multiple_processes(commands, env=clean_env())
+  logger.info('compiled %d inputs' % len(commands))
 
 
 def create_lib(libname, inputs):
@@ -92,6 +101,115 @@ def create_lib(libname, inputs):
     building.emar('cr', libname, inputs)
 
 
+def run_ninja(build_dir):
+  diagnostics.warning('experimental', 'ninja support is experimental')
+  cmd = ['ninja', '-C', build_dir]
+  if shared.PRINT_STAGES:
+    cmd.append('-v')
+  shared.check_call(cmd, env=clean_env())
+
+
+def create_ninja_file(input_files, filename, libname, cflags, asflags=None, customize_build_flags=None):
+  if asflags is None:
+    asflags = []
+  # TODO(sbc) There is an llvm bug that causes a crash when `-g` is used with
+  # assembly files that define wasm globals.
+  asflags = [arg for arg in asflags if arg != '-g']
+  cflags_asm = [arg for arg in cflags if arg != '-g']
+
+  def join(flags):
+    return ' '.join(flags)
+
+  out = f'''\
+# Automatically generated by tools/system_libs.py.  DO NOT EDIT
+
+ninja_required_version = 1.5
+
+ASFLAGS = {join(asflags)}
+CFLAGS = {join(cflags)}
+CFLAGS_ASM = {join(cflags_asm)}
+EMCC = {shared.EMCC}
+EMXX = {shared.EMXX}
+EMAR = {shared.EMAR}
+
+rule cc
+  depfile = $out.d
+  command = $EMCC -MD -MF $out.d $CFLAGS -c $in -o $out
+  description = CC $out
+
+rule cxx
+  depfile = $out.d
+  command = $EMXX -MD -MF $out.d $CFLAGS -c $in -o $out
+  description = CXX $out
+
+rule asm
+  command = $EMCC $ASFLAGS -c $in -o $out
+  description = ASM $out
+
+rule asm_cpp
+  depfile = $out.d
+  command = $EMCC -MD -MF $out.d $CFLAGS_ASM -c $in -o $out
+  description = ASM $out
+
+rule direct_cc
+  depfile = $with_depfile
+  command = $EMCC -MD -MF $with_depfile $CFLAGS -c $in -o $out
+  description = CC $out
+
+rule archive
+  command = $EMAR cr $out $in
+  description = AR $out
+
+'''
+  suffix = shared.suffix(libname)
+
+  case_insensitive = is_case_insensitive(os.path.dirname(filename))
+  if suffix == '.o':
+    assert len(input_files) == 1
+    depfile = shared.unsuffixed_basename(input_files[0]) + '.d'
+    out += f'build {libname}: direct_cc {input_files[0]}\n'
+    out += f'  with_depfile = {depfile}\n'
+  else:
+    objects = []
+    for src in input_files:
+      # Resolve duplicates by appending unique.
+      # This is needed on case insensitve filesystem to handle,
+      # for example, _exit.o and _Exit.o.
+      o = shared.unsuffixed_basename(src) + '.o'
+      object_uuid = 0
+      if case_insensitive:
+        o = o.lower()
+      # Find a unique basename
+      while o in objects:
+        object_uuid += 1
+        o = f'{o}__{object_uuid}.o'
+      objects.append(o)
+      ext = shared.suffix(src)
+      if ext == '.s':
+        out += f'build {o}: asm {src}\n'
+        flags = asflags
+      elif ext == '.S':
+        out += f'build {o}: asm_cpp {src}\n'
+        flags = cflags_asm
+      elif ext == '.c':
+        out += f'build {o}: cc {src}\n'
+        flags = cflags
+      else:
+        out += f'build {o}: cxx {src}\n'
+        flags = cflags
+      if customize_build_flags:
+        custom_flags = customize_build_flags(flags, src)
+        if custom_flags != flags:
+          out += f'  CFLAGS = {join(custom_flags)}'
+      out += '\n'
+
+    objects = sorted(objects, key=lambda x: os.path.basename(x))
+    objects = ' '.join(objects)
+    out += f'build {libname}: archive {objects}\n'
+
+  utils.write_file(filename, out)
+
+
 def is_case_insensitive(path):
   """Returns True if the filesystem at `path` is case insensitive."""
   utils.write_file(os.path.join(path, 'test_file'), '')
@@ -239,15 +357,18 @@ class Library:
     return True
 
   def erase(self):
-    shared.Cache.erase_file(shared.Cache.get_lib_name(self.get_filename()))
+    shared.Cache.erase_file(self.get_path())
 
-  def get_path(self):
+  def get_path(self, absolute=False):
+    return shared.Cache.get_lib_name(self.get_filename(), absolute=absolute)
+
+  def build(self):
     """
     Gets the cached path of this library.
 
     This will trigger a build if this library is not in the cache.
     """
-    return shared.Cache.get_lib(self.get_filename(), self.build)
+    return shared.Cache.get(self.get_path(), self.do_build, force=USE_NINJA == 2, quiet=USE_NINJA)
 
   def get_link_flag(self):
     """
@@ -255,7 +376,7 @@ class Library:
 
     This will trigger a build if this library is not in the cache.
     """
-    fullpath = self.get_path()
+    fullpath = self.build()
     # For non-libaries (e.g. crt1.o) we pass the entire path to the linker
     if self.get_ext() != '.a':
       return fullpath
@@ -281,6 +402,17 @@ class Library:
 
     raise NotImplementedError()
 
+  def build_with_ninja(self, build_dir, libname):
+    ensure_sysroot()
+    utils.safe_ensure_dirs(build_dir)
+
+    cflags = self.get_cflags()
+    asflags = get_base_cflags()
+    input_files = self.get_files()
+    ninja_file = os.path.join(build_dir, 'build.ninja')
+    create_ninja_file(input_files, ninja_file, libname, cflags, asflags=asflags, customize_build_flags=self.customize_build_cmd)
+    run_ninja(build_dir)
+
   def build_objects(self, build_dir):
     """
     Returns a list of compiled object files for this library.
@@ -311,7 +443,13 @@ class Library:
       else:
         cmd = [shared.EMXX]
 
-      cmd += cflags
+      if ext == '.s':
+        # .s files are processed directly by the assembler.  In this case we can't pass
+        # pre-processor flags such as `-I` and `-D` but we still want core flags such as
+        # `-sMEMORY64`.
+        cmd += get_base_cflags()
+      else:
+        cmd += cflags
       if ext in ('.s', '.S'):
         # TODO(sbc) There is an llvm bug that causes a crash when `-g` is used with
         # assembly files that define wasm globals.
@@ -328,13 +466,20 @@ class Library:
     For example, libc uses this to replace -Oz with -O2 for some subset of files."""
     return cmd
 
-  def build(self, out_filename):
+  def do_build(self, out_filename):
     """Builds the library and returns the path to the file."""
-    build_dir = shared.Cache.get_path(os.path.join('build', self.get_base_name()))
-    utils.safe_ensure_dirs(build_dir)
-    create_lib(out_filename, self.build_objects(build_dir))
-    if not shared.DEBUG:
-      utils.delete_dir(build_dir)
+    assert out_filename == self.get_path(absolute=True)
+    build_dir = os.path.join(shared.Cache.get_path('build'), self.get_base_name())
+    if USE_NINJA:
+      self.build_with_ninja(build_dir, out_filename)
+    else:
+      # Use a seperate build directory to the ninja flavor so that building without
+      # EMCC_USE_NINJA doesn't clobber the ninja build tree
+      build_dir += '-tmp'
+      utils.safe_ensure_dirs(build_dir)
+      create_lib(out_filename, self.build_objects(build_dir))
+      if not shared.DEBUG:
+        utils.delete_dir(build_dir)
 
   @classmethod
   def _inherit_list(cls, attr):
@@ -623,10 +768,6 @@ class SjLjLibrary(Library):
     is_wasm = settings.SUPPORT_LONGJMP == 'wasm'
     return super().get_default_variation(is_wasm=is_wasm, **kwargs)
 
-  def can_build(self):
-    # wasm-sjlj is not yet supported with MEMORY64
-    return not (settings.MEMORY64 and self.is_wasm)
-
 
 class MuslInternalLibrary(Library):
   includes = [
@@ -678,7 +819,7 @@ class libcompiler_rt(MTLibrary, SjLjLibrary):
   cflags = ['-fno-builtin']
   src_dir = 'system/lib/compiler-rt/lib/builtins'
   # gcc_personality_v0.c depends on libunwind, which don't include by default.
-  src_files = glob_in_path(src_dir, '*.c', excludes=['gcc_personality_v0.c'])
+  src_files = glob_in_path(src_dir, '*.c', excludes=['gcc_personality_v0.c', 'truncdfbf2.c', 'truncsfbf2.c'])
   src_files += files_in_path(
       path='system/lib/compiler-rt',
       filenames=[
@@ -686,6 +827,7 @@ class libcompiler_rt(MTLibrary, SjLjLibrary):
         'stack_limits.S',
         'emscripten_setjmp.c',
         'emscripten_exception_builtins.c',
+        'emscripten_tempret.s',
         '__trap.c',
       ])
 
@@ -709,7 +851,10 @@ class libc(MuslInternalLibrary,
   # functions to something else based on assumptions that they behave exactly
   # like the standard library. This can cause unexpected bugs when we use our
   # custom standard library. The same for other libc/libm builds.
-  cflags = ['-Os', '-fno-builtin']
+  # We use -fno-inline-functions because it can produce slightly smaller
+  # (and slower) code in some cases.  TODO(sbc): remove this and let llvm weight
+  # the cost/benefit of inlining.
+  cflags = ['-Os', '-fno-inline-functions', '-fno-builtin']
 
   # Disable certain warnings for code patterns that are contained in upstream musl
   cflags += ['-Wno-ignored-attributes',
@@ -807,6 +952,7 @@ class libc(MuslInternalLibrary,
         'alarm.c', 'syscall.c', 'popen.c', 'pclose.c',
         'getgrouplist.c', 'initgroups.c', 'wordexp.c', 'timer_create.c',
         'getentropy.c',
+        'getauxval.c',
         # 'process' exclusion
         'fork.c', 'vfork.c', 'posix_spawn.c', 'posix_spawnp.c', 'execve.c', 'waitid.c', 'system.c',
         '_Fork.c',
@@ -1018,7 +1164,7 @@ class libc(MuslInternalLibrary,
 class libc_optz(libc):
   name = 'libc_optz'
 
-  cflags = ['-Os', '-fno-builtin', '-DEMSCRIPTEN_OPTIMIZE_FOR_OZ']
+  cflags = ['-Os', '-fno-inline-functions', '-fno-builtin', '-DEMSCRIPTEN_OPTIMIZE_FOR_OZ']
 
   def __init__(self, **kwargs):
     super().__init__(**kwargs)
@@ -1124,7 +1270,7 @@ class libwasm_workers(MTLibrary):
 class libsockets(MuslInternalLibrary, MTLibrary):
   name = 'libsockets'
 
-  cflags = ['-Os', '-fno-builtin', '-Wno-shift-op-parentheses']
+  cflags = ['-Os', '-fno-inline-functions', '-fno-builtin', '-Wno-shift-op-parentheses']
 
   def get_files(self):
     return files_in_path(
@@ -1138,7 +1284,7 @@ class libsockets(MuslInternalLibrary, MTLibrary):
 class libsockets_proxy(MTLibrary):
   name = 'libsockets_proxy'
 
-  cflags = ['-Os']
+  cflags = ['-Os', '-fno-inline-functions']
 
   def get_files(self):
     return [utils.path_from_root('system/lib/websocket/websocket_to_posix_socket.c')]
@@ -1208,8 +1354,10 @@ class libcxxabi(NoExceptLibrary, MTLibrary):
   name = 'libc++abi'
   cflags = [
       '-Oz',
+      '-fno-inline-functions',
       '-D_LIBCXXABI_BUILDING_LIBRARY',
       '-DLIBCXXABI_NON_DEMANGLING_TERMINATE',
+      '-std=c++14',
     ]
   includes = ['system/lib/libcxx/src']
 
@@ -1266,6 +1414,7 @@ class libcxx(NoExceptLibrary, MTLibrary):
 
   cflags = [
     '-Oz',
+    '-fno-inline-functions',
     '-DLIBCXX_BUILDING_LIBCXXABI=1',
     '-D_LIBCPP_BUILDING_LIBRARY',
     '-D_LIBCPP_DISABLE_VISIBILITY_ANNOTATIONS',
@@ -1298,7 +1447,7 @@ class libunwind(NoExceptLibrary, MTLibrary):
   # See https://bugs.llvm.org/show_bug.cgi?id=44353
   force_object_files = True
 
-  cflags = ['-Oz', '-D_LIBUNWIND_DISABLE_VISIBILITY_ANNOTATIONS']
+  cflags = ['-Oz', '-fno-inline-functions', '-D_LIBUNWIND_DISABLE_VISIBILITY_ANNOTATIONS']
   src_dir = 'system/lib/libunwind/src'
   # Without this we can't build libunwind since it will pickup the unwind.h
   # that is part of llvm (which is not compatible for some reason).
@@ -1329,6 +1478,9 @@ class libmalloc(MTLibrary):
   name = 'libmalloc'
 
   cflags = ['-fno-builtin']
+  # malloc/free/calloc are runtime functions and can be generated during LTO
+  # Therefor they cannot themselves be part of LTO.
+  force_object_files = True
 
   def __init__(self, **kwargs):
     self.malloc = kwargs.pop('malloc')
@@ -1384,11 +1536,6 @@ class libmalloc(MTLibrary):
   def can_use(self):
     return super().can_use() and settings.MALLOC != 'none'
 
-  def can_build(self):
-    # emmalloc is not currently compatible with 64-bit pointers.  See
-    # the comment at the top of emmalloc.c.
-    return not self.malloc.startswith('emmalloc') or not settings.MEMORY64
-
   @classmethod
   def vary_on(cls):
     return super().vary_on() + ['is_debug', 'use_errno', 'is_tracing', 'memvalidate', 'verbose']
@@ -1418,7 +1565,7 @@ class libmalloc(MTLibrary):
 class libal(Library):
   name = 'libal'
 
-  cflags = ['-Os']
+  cflags = ['-Os', '-fno-inline-functions']
   src_dir = 'system/lib'
   src_files = ['al.c']
 
@@ -1429,7 +1576,7 @@ class libGL(MTLibrary):
   src_dir = 'system/lib/gl'
   src_files = ['gl.c', 'webgl1.c', 'libprocaddr.c']
 
-  cflags = ['-Oz']
+  cflags = ['-Oz', '-fno-inline-functions']
 
   def __init__(self, **kwargs):
     self.is_legacy = kwargs.pop('is_legacy')
@@ -1545,10 +1692,35 @@ class libwasmfs(DebugLibrary, AsanInstrumentedLibrary, MTLibrary):
 
   includes = ['system/lib/wasmfs', 'system/lib/pthread']
 
+  def __init__(self, **kwargs):
+    self.ignore_case = kwargs.pop('ignore_case')
+    super().__init__(**kwargs)
+
+  def get_cflags(self):
+    cflags = super().get_cflags()
+    if self.ignore_case:
+      cflags += ['-DWASMFS_CASE_INSENSITIVE']
+    return cflags
+
+  def get_base_name(self):
+    name = super().get_base_name()
+    if self.ignore_case:
+      name += '-icase'
+    return name
+
+  @classmethod
+  def vary_on(cls):
+    return super().vary_on() + ['ignore_case']
+
+  @classmethod
+  def get_default_variation(cls, **kwargs):
+    return super().get_default_variation(ignore_case=settings.CASE_INSENSITIVE_FS, **kwargs)
+
   def get_files(self):
     backends = files_in_path(
         path='system/lib/wasmfs/backends',
         filenames=['fetch_backend.cpp',
+                   'ignore_case_backend.cpp',
                    'js_file_backend.cpp',
                    'memory_backend.cpp',
                    'node_backend.cpp',
@@ -1572,7 +1744,7 @@ class libwasmfs(DebugLibrary, AsanInstrumentedLibrary, MTLibrary):
 class libhtml5(Library):
   name = 'libhtml5'
 
-  cflags = ['-Oz']
+  cflags = ['-Oz', '-fno-inline-functions']
   src_dir = 'system/lib/html5'
   src_glob = '*.c'
 
@@ -1617,6 +1789,7 @@ class libubsan_rt(SanitizerLibrary):
 
   cflags = ['-DUBSAN_CAN_USE_CXXABI']
   src_dir = 'system/lib/compiler-rt/lib/ubsan'
+  src_glob_exclude = ['ubsan_diag_standalone.cpp']
 
 
 class liblsan_common_rt(SanitizerLibrary):
@@ -1660,7 +1833,7 @@ class libstandalonewasm(MuslInternalLibrary):
   # LTO defeats the weak linking trick used in __original_main.c
   force_object_files = True
 
-  cflags = ['-Os', '-fno-builtin']
+  cflags = ['-Os', '-fno-inline-functions', '-fno-builtin']
   src_dir = 'system/lib'
 
   def __init__(self, **kwargs):
@@ -1730,7 +1903,7 @@ class libstandalonewasm(MuslInternalLibrary):
 
 class libjsmath(Library):
   name = 'libjsmath'
-  cflags = ['-Os']
+  cflags = ['-Os', '-fno-inline-functions']
   src_dir = 'system/lib'
   src_files = ['jsmath.c']
 
@@ -1759,7 +1932,7 @@ def warn_on_unexported_main(symbolses):
 
 
 def handle_reverse_deps(input_files):
-  if settings.REVERSE_DEPS == 'none':
+  if settings.REVERSE_DEPS == 'none' or settings.SIDE_MODULE:
     return
   elif settings.REVERSE_DEPS == 'all':
     # When not optimzing we add all possible reverse dependencies rather
@@ -1864,8 +2037,31 @@ def get_libs_to_link(args, forced, only_forced):
   if '-nodefaultlibs' in args:
     return libs_to_link
 
+  sanitize = settings.USE_LSAN or settings.USE_ASAN or settings.UBSAN_RUNTIME
+
+  def add_sanitizer_libs():
+    if settings.USE_ASAN:
+      force_include.append('libasan_rt')
+      add_library('libasan_rt')
+      add_library('libasan_js')
+    elif settings.USE_LSAN:
+      force_include.append('liblsan_rt')
+      add_library('liblsan_rt')
+
+    if settings.UBSAN_RUNTIME == 1:
+      add_library('libubsan_minimal_rt')
+    elif settings.UBSAN_RUNTIME == 2:
+      add_library('libubsan_rt')
+
+    if settings.USE_LSAN or settings.USE_ASAN:
+      add_library('liblsan_common_rt')
+
+    if sanitize:
+      add_library('libsanitizer_common_rt')
+
   if only_forced:
     add_library('libcompiler_rt')
+    add_sanitizer_libs()
     return libs_to_link
 
   if settings.AUTO_NATIVE_LIBRARIES:
@@ -1873,8 +2069,6 @@ def get_libs_to_link(args, forced, only_forced):
     add_library('libal')
     add_library('libhtml5')
 
-  sanitize = settings.USE_LSAN or settings.USE_ASAN or settings.UBSAN_RUNTIME
-
   # JS math must come before anything else, so that it overrides the normal
   # libc math.
   if settings.JS_MATH:
@@ -1906,25 +2100,6 @@ def get_libs_to_link(args, forced, only_forced):
     if settings.WASM_EXCEPTIONS:
       add_library('libunwind')
 
-  if settings.USE_ASAN:
-    force_include.append('libasan_rt')
-    add_library('libasan_rt')
-    add_library('libasan_js')
-  elif settings.USE_LSAN:
-    force_include.append('liblsan_rt')
-    add_library('liblsan_rt')
-
-  if settings.UBSAN_RUNTIME == 1:
-    add_library('libubsan_minimal_rt')
-  elif settings.UBSAN_RUNTIME == 2:
-    add_library('libubsan_rt')
-
-  if settings.USE_LSAN or settings.USE_ASAN:
-    add_library('liblsan_common_rt')
-
-  if sanitize:
-    add_library('libsanitizer_common_rt')
-
   if settings.PROXY_POSIX_SOCKETS:
     add_library('libsockets_proxy')
   else:
@@ -1936,6 +2111,7 @@ def get_libs_to_link(args, forced, only_forced):
   if settings.WASM_WORKERS:
     add_library('libwasm_workers')
 
+  add_sanitizer_libs()
   return libs_to_link
 
 
