@@ -3,21 +3,22 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-'''
-WebIDL binder
+"""WebIDL binder
 
-http://kripken.github.io/emscripten-site/docs/porting/connecting_cpp_and_javascript/WebIDL-Binder.html
-'''
+https://emscripten.org/docs/porting/connecting_cpp_and_javascript/WebIDL-Binder.html
+"""
 
-from __future__ import print_function
-import os, sys
+import os
+import sys
 
-sys.path.insert(1, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+__scriptdir__ = os.path.dirname(os.path.abspath(__file__))
+__rootdir__ = os.path.dirname(__scriptdir__)
+sys.path.append(__rootdir__)
 
-from tools import shared
+from tools import utils
 
-sys.path.append(shared.path_from_root('third_party'))
-sys.path.append(shared.path_from_root('third_party', 'ply'))
+sys.path.append(utils.path_from_root('third_party'))
+sys.path.append(utils.path_from_root('third_party/ply'))
 
 import WebIDL
 
@@ -26,34 +27,37 @@ import WebIDL
 # CHECKS='ALL' will do extensive argument type checking (~5x slower than default).
 #                 This will catch invalid numbers, invalid pointers, invalid strings, etc.
 # Anything else defaults to legacy mode for backward compatibility.
-CHECKS = os.environ.get('IDL_CHECKS') or 'DEFAULT'
+CHECKS = os.environ.get('IDL_CHECKS', 'DEFAULT')
 # DEBUG=1 will print debug info in render_function
-DEBUG = os.environ.get('IDL_VERBOSE') is '1'
+DEBUG = os.environ.get('IDL_VERBOSE') == '1'
 
-if DEBUG: print("Debug print ON, CHECKS=%s" % CHECKS)
+if DEBUG:
+  print("Debug print ON, CHECKS=%s" % CHECKS)
 
 # We need to avoid some closure errors on the constructors we define here.
-CONSTRUCTOR_CLOSURE_SUPPRESSIONS = '/** @suppress {undefinedVars, duplicate} */'
+CONSTRUCTOR_CLOSURE_SUPPRESSIONS = '/** @suppress {undefinedVars, duplicate} @this{Object} */'
 
-class Dummy(object):
+
+class Dummy:
   def __init__(self, init):
     for k, v in init.items():
       self.__dict__[k] = v
 
-  def getExtendedAttribute(self, name):
+  def getExtendedAttribute(self, name): # noqa: U100
     return None
 
+
 input_file = sys.argv[1]
 output_base = sys.argv[2]
 
-shared.try_delete(output_base + '.cpp')
-shared.try_delete(output_base + '.js')
+utils.delete_file(output_base + '.cpp')
+utils.delete_file(output_base + '.js')
 
 p = WebIDL.Parser()
 p.parse(r'''
 interface VoidPtr {
 };
-''' + open(input_file).read())
+''' + utils.read_file(input_file))
 data = p.finish()
 
 interfaces = {}
@@ -68,8 +72,8 @@ for thing in data:
   elif isinstance(thing, WebIDL.IDLEnum):
     enums[thing.identifier.name] = thing
 
-#print interfaces
-#print implements
+# print interfaces
+# print implements
 
 pre_c = []
 mid_c = []
@@ -83,6 +87,7 @@ mid_c += [r'''
 extern "C" {
 ''']
 
+
 def build_constructor(name):
   implementing_name = implements[name][0] if implements.get(name) else 'WrapperObject'
   return [r'''{name}.prototype = Object.create({implementing}.prototype);
@@ -96,6 +101,7 @@ Module['{name}'] = {name};
 mid_js += ['''
 // Bindings utilities
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function WrapperObject() {
 }
 ''']
@@ -103,11 +109,15 @@ function WrapperObject() {
 mid_js += build_constructor('WrapperObject')
 
 mid_js += ['''
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant)
+    @param {*=} __class__ */
 function getCache(__class__) {
   return (__class__ || WrapperObject).__cache__;
 }
 Module['getCache'] = getCache;
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant)
+    @param {*=} __class__ */
 function wrapPointer(ptr, __class__) {
   var cache = getCache(__class__);
   var ret = cache[ptr];
@@ -118,6 +128,7 @@ function wrapPointer(ptr, __class__) {
 }
 Module['wrapPointer'] = wrapPointer;
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function castObject(obj, __class__) {
   return wrapPointer(obj.ptr, __class__);
 }
@@ -125,6 +136,7 @@ Module['castObject'] = castObject;
 
 Module['NULL'] = wrapPointer(0);
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function destroy(obj) {
   if (!obj['__destroy__']) throw 'Error: Cannot destroy object. (Did you create it yourself?)';
   obj['__destroy__']();
@@ -133,16 +145,19 @@ function destroy(obj) {
 }
 Module['destroy'] = destroy;
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function compare(obj1, obj2) {
   return obj1.ptr === obj2.ptr;
 }
 Module['compare'] = compare;
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function getPointer(obj) {
   return obj.ptr;
 }
 Module['getPointer'] = getPointer;
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function getClass(obj) {
   return obj.__class__;
 }
@@ -150,6 +165,7 @@ Module['getClass'] = getClass;
 
 // Converts big (string or array) values into a C-style storage, in temporary space
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 var ensureCache = {
   buffer: 0,  // the main buffer of temporary storage
   size: 0,   // the size of buffer
@@ -198,19 +214,20 @@ var ensureCache = {
     return ret;
   },
   copy: function(array, view, offset) {
-    var offsetShifted = offset;
+    offset >>>= 0;
     var bytes = view.BYTES_PER_ELEMENT;
     switch (bytes) {
-      case 2: offsetShifted >>= 1; break;
-      case 4: offsetShifted >>= 2; break;
-      case 8: offsetShifted >>= 3; break;
+      case 2: offset >>>= 1; break;
+      case 4: offset >>>= 2; break;
+      case 8: offset >>>= 3; break;
     }
     for (var i = 0; i < array.length; i++) {
-      view[offsetShifted + i] = array[i];
+      view[offset + i] = array[i];
     }
   },
 };
 
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function ensureString(value) {
   if (typeof value === 'string') {
     var intArray = intArrayFromString(value);
@@ -220,6 +237,7 @@ function ensureString(value) {
   }
   return value;
 }
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function ensureInt8(value) {
   if (typeof value === 'object') {
     var offset = ensureCache.alloc(value, HEAP8);
@@ -228,6 +246,7 @@ function ensureInt8(value) {
   }
   return value;
 }
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function ensureInt16(value) {
   if (typeof value === 'object') {
     var offset = ensureCache.alloc(value, HEAP16);
@@ -236,6 +255,7 @@ function ensureInt16(value) {
   }
   return value;
 }
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function ensureInt32(value) {
   if (typeof value === 'object') {
     var offset = ensureCache.alloc(value, HEAP32);
@@ -244,6 +264,7 @@ function ensureInt32(value) {
   }
   return value;
 }
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function ensureFloat32(value) {
   if (typeof value === 'object') {
     var offset = ensureCache.alloc(value, HEAPF32);
@@ -252,6 +273,7 @@ function ensureFloat32(value) {
   }
   return value;
 }
+/** @suppress {duplicate} (TODO: avoid emitting this multiple times, it is redundant) */
 function ensureFloat64(value) {
   if (typeof value === 'object') {
     var offset = ensureCache.alloc(value, HEAPF64);
@@ -265,22 +287,27 @@ function ensureFloat64(value) {
 
 mid_c += ['''
 // Not using size_t for array indices as the values used by the javascript code are signed.
+
+EM_JS(void, array_bounds_check_error, (size_t idx, size_t size), {
+  throw 'Array index ' + idx + ' out of bounds: [0,' + size + ')';
+});
+
 void array_bounds_check(const int array_size, const int array_idx) {
   if (array_idx < 0 || array_idx >= array_size) {
-    EM_ASM({
-      throw 'Array index ' + $0 + ' out of bounds: [0,' + $1 + ')';
-    }, array_idx, array_size);
+    array_bounds_check_error(array_idx, array_size);
   }
 }
 ''']
 
 C_FLOATS = ['float', 'double']
 
+
 def full_typename(arg):
   return ('const ' if arg.getExtendedAttribute('Const') else '') + arg.type.name + ('[]' if arg.type.isArray() else '')
 
+
 def type_to_c(t, non_pointing=False):
-  #print 'to c ', t
+  # print 'to c ', t
   def base_type_to_c(t):
     if t == 'Long':
       ret = 'int'
@@ -328,31 +355,37 @@ def type_to_c(t, non_pointing=False):
     prefix = 'const '
   return prefix + base_type_to_c(t) + suffix
 
+
 def take_addr_if_nonpointer(m):
   if m.getExtendedAttribute('Ref') or m.getExtendedAttribute('Value'):
     return '&'
   return ''
 
+
 def deref_if_nonpointer(m):
   if m.getExtendedAttribute('Ref') or m.getExtendedAttribute('Value'):
     return '*'
   return ''
 
+
 def type_to_cdec(raw):
-  name = ret = type_to_c(raw.type.name, non_pointing=True)
-  if raw.getExtendedAttribute('Const'): ret = 'const ' + ret
-  if raw.type.name not in interfaces: return ret
+  ret = type_to_c(raw.type.name, non_pointing=True)
+  if raw.getExtendedAttribute('Const'):
+    ret = 'const ' + ret
+  if raw.type.name not in interfaces:
+    return ret
   if raw.getExtendedAttribute('Ref'):
     return ret + '&'
   if raw.getExtendedAttribute('Value'):
     return ret
   return ret + '*'
 
-def render_function(class_name, func_name, sigs, return_type, non_pointer, copy, operator, constructor, func_scope, call_content=None, const=False, array_attribute=False):
-  global mid_c, mid_js, js_impl_methods
 
+def render_function(class_name, func_name, sigs, return_type, non_pointer,
+                    copy, operator, constructor, func_scope,
+                    call_content=None, const=False, array_attribute=False):
   legacy_mode = CHECKS not in ['ALL', 'FAST']
-  all_checks = CHECKS  == 'ALL'
+  all_checks = CHECKS == 'ALL'
 
   bindings_name = class_name + '_' + func_name
   min_args = min(sigs.keys())
@@ -374,7 +407,8 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
   cache = ('getCache(%s)[this.ptr] = this;' % class_name) if constructor else ''
   call_prefix = '' if not constructor else 'this.ptr = '
   call_postfix = ''
-  if return_type != 'Void' and not constructor: call_prefix = 'return '
+  if return_type != 'Void' and not constructor:
+    call_prefix = 'return '
   if not constructor:
     if return_type in interfaces:
       call_prefix += 'wrapPointer('
@@ -424,23 +458,29 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
       body += "  /* %s <%s> [%s] */\n" % (js_arg, arg.type.name, inner)
 
       # Wrap asserts with existence check when argument is optional.
-      if all_checks and optional: body += "if(typeof {0} !== 'undefined' && {0} !== null) {{\n".format(js_arg)
+      if all_checks and optional:
+        body += "if(typeof {0} !== 'undefined' && {0} !== null) {{\n".format(js_arg)
       # Special case argument types.
       if arg.type.isNumeric():
         if arg.type.isInteger():
-          if all_checks: body += "  assert(typeof {0} === 'number' && !isNaN({0}), '{1}Expecting <integer>');\n".format(js_arg, check_msg)
+          if all_checks:
+            body += "  assert(typeof {0} === 'number' && !isNaN({0}), '{1}Expecting <integer>');\n".format(js_arg, check_msg)
         else:
-          if all_checks: body += "  assert(typeof {0} === 'number', '{1}Expecting <number>');\n".format(js_arg, check_msg)
+          if all_checks:
+            body += "  assert(typeof {0} === 'number', '{1}Expecting <number>');\n".format(js_arg, check_msg)
         # No transform needed for numbers
       elif arg.type.isBoolean():
-        if all_checks: body += "  assert(typeof {0} === 'boolean' || (typeof {0} === 'number' && !isNaN({0})), '{1}Expecting <boolean>');\n".format(js_arg, check_msg)
+        if all_checks:
+          body += "  assert(typeof {0} === 'boolean' || (typeof {0} === 'number' && !isNaN({0})), '{1}Expecting <boolean>');\n".format(js_arg, check_msg)
         # No transform needed for booleans
       elif arg.type.isString():
         # Strings can be DOM strings or pointers.
-        if all_checks: body += "  assert(typeof {0} === 'string' || ({0} && typeof {0} === 'object' && typeof {0}.ptr === 'number'), '{1}Expecting <string>');\n".format(js_arg, check_msg)
+        if all_checks:
+          body += "  assert(typeof {0} === 'string' || ({0} && typeof {0} === 'object' && typeof {0}.ptr === 'number'), '{1}Expecting <string>');\n".format(js_arg, check_msg)
         do_default = True # legacy path is fast enough for strings.
       elif arg.type.isInterface():
-        if all_checks: body += "  assert(typeof {0} === 'object' && typeof {0}.ptr === 'number', '{1}Expecting <pointer>');\n".format(js_arg, check_msg)
+        if all_checks:
+          body += "  assert(typeof {0} === 'object' && typeof {0}.ptr === 'number', '{1}Expecting <pointer>');\n".format(js_arg, check_msg)
         if optional:
           body += "  if(typeof {0} !== 'undefined' && {0} !== null) {{ {0} = {0}.ptr }};\n".format(js_arg)
         else:
@@ -449,7 +489,8 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
       else:
         do_default = True
 
-      if all_checks and optional: body += "}\n"
+      if all_checks and optional:
+        body += "}\n"
     else:
       do_default = True
 
@@ -480,15 +521,16 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
   body += '  %s%s(%s)%s;\n' % (call_prefix, '_' + c_names[max_args], ', '.join(pre_arg + args), call_postfix)
   if cache:
     body += '  ' + cache + '\n'
-  mid_js += [r'''%sfunction%s(%s) {
+  mid_js.append(r'''%sfunction%s(%s) {
 %s
-};''' % (CONSTRUCTOR_CLOSURE_SUPPRESSIONS, (' ' + func_name) if constructor else '', ', '.join(args), body[:-1])]
+};''' % (CONSTRUCTOR_CLOSURE_SUPPRESSIONS, (' ' + func_name) if constructor else '', ', '.join(args), body[:-1]))
 
   # C
 
-  for i in range(min_args, max_args+1):
+  for i in range(min_args, max_args + 1):
     raw = sigs.get(i)
-    if raw is None: continue
+    if raw is None:
+      continue
     sig = list(map(full_typename, raw))
     if array_attribute:
       sig = [x.replace('[]', '') for x in sig] # for arrays, ignore that this is an array - our get/set methods operate on the elements
@@ -525,11 +567,11 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
 
     pre = ''
 
-    basic_return = 'return ' if constructor or return_type is not 'Void' else ''
+    basic_return = 'return ' if constructor or return_type != 'Void' else ''
     return_prefix = basic_return
     return_postfix = ''
     if non_pointer:
-      return_prefix += '&';
+      return_prefix += '&'
     if copy:
       pre += '  static %s temp;\n' % type_to_c(return_type, non_pointing=True)
       return_prefix += '(temp = '
@@ -537,23 +579,23 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
 
     c_return_type = type_to_c(return_type)
     maybe_const = 'const ' if const else ''
-    mid_c += [r'''
+    mid_c.append(r'''
 %s%s EMSCRIPTEN_KEEPALIVE %s(%s) {
 %s  %s%s%s;
 }
-''' % (maybe_const, type_to_c(class_name) if constructor else c_return_type, c_names[i], full_args, pre, return_prefix, call, return_postfix)]
+''' % (maybe_const, type_to_c(class_name) if constructor else c_return_type, c_names[i], full_args, pre, return_prefix, call, return_postfix))
 
     if not constructor:
       if i == max_args:
         dec_args = ', '.join([type_to_cdec(raw[j]) + ' ' + args[j] for j in range(i)])
-        js_call_args = ', '.join(['%s%s' % (('(int)' if sig[j] in interfaces else '') + take_addr_if_nonpointer(raw[j]), args[j]) for j in range(i)])
+        js_call_args = ', '.join(['%s%s' % (('(ptrdiff_t)' if sig[j] in interfaces else '') + take_addr_if_nonpointer(raw[j]), args[j]) for j in range(i)])
 
-        js_impl_methods += [r'''  %s %s(%s) %s {
+        js_impl_methods.append(r'''  %s %s(%s) %s {
     %sEM_ASM_%s({
       var self = Module['getCache'](Module['%s'])[$0];
       if (!self.hasOwnProperty('%s')) throw 'a JSImplementation must implement all functions, you forgot %s::%s.';
       %sself['%s'](%s)%s;
-    }, (int)this%s);
+    }, (ptrdiff_t)this%s);
   }''' % (c_return_type, func_name, dec_args, maybe_const,
           basic_return, 'INT' if c_return_type not in C_FLOATS else 'DOUBLE',
           class_name,
@@ -562,12 +604,13 @@ def render_function(class_name, func_name, sigs, return_type, non_pointer, copy,
           func_name,
           ','.join(['$%d' % i for i in range(1, max_args + 1)]),
           return_postfix,
-          (', ' if js_call_args else '') + js_call_args)]
+          (', ' if js_call_args else '') + js_call_args))
 
 
 for name, interface in interfaces.items():
   js_impl = interface.getExtendedAttribute('JSImplementation')
-  if not js_impl: continue
+  if not js_impl:
+    continue
   implements[name] = [js_impl[0]]
 
 # Compute the height in the inheritance tree of each node. Note that the order of interation
@@ -598,11 +641,11 @@ for name in names:
   mid_js += ['\n// ' + name + '\n']
   mid_c += ['\n// ' + name + '\n']
 
-  global js_impl_methods
   js_impl_methods = []
 
   cons = interface.getExtendedAttribute('Constructor')
-  if type(cons) == list: raise Exception('do not use "Constructor", instead create methods with the name of the interface')
+  if type(cons) == list:
+    raise Exception('do not use "Constructor", instead create methods with the name of the interface')
 
   js_impl = interface.getExtendedAttribute('JSImplementation')
   if js_impl:
@@ -616,7 +659,8 @@ for name in names:
     mid_js += build_constructor(name)
 
   for m in interface.members:
-    if not m.isMethod(): continue
+    if not m.isMethod():
+      continue
     constructor = m.identifier.name == name
     if not constructor:
       parent_constructor = False
@@ -637,7 +681,7 @@ for name in names:
         return_type = ret.name
       else:
         assert return_type == ret.name, 'overloads must have the same return type'
-      for i in range(len(args)+1):
+      for i in range(len(args) + 1):
         if i == len(args) or args[i].optional:
           assert i not in sigs, 'overloading must differentiate by # of arguments (cannot have two signatures that differ by types but not by length)'
           sigs[i] = args[:i]
@@ -654,13 +698,14 @@ for name in names:
       mid_js += build_constructor(name)
 
   for m in interface.members:
-    if not m.isAttr(): continue
+    if not m.isAttr():
+      continue
     attr = m.identifier.name
 
     if m.type.isArray():
-      get_sigs = { 1: [Dummy({ 'type': WebIDL.BuiltinTypes[WebIDL.IDLBuiltinType.Types.long] })] }
-      set_sigs = { 2: [Dummy({ 'type': WebIDL.BuiltinTypes[WebIDL.IDLBuiltinType.Types.long] }),
-                       Dummy({ 'type': m.type })] }
+      get_sigs = {1: [Dummy({'type': WebIDL.BuiltinTypes[WebIDL.IDLBuiltinType.Types.long]})]}
+      set_sigs = {2: [Dummy({'type': WebIDL.BuiltinTypes[WebIDL.IDLBuiltinType.Types.long]}),
+                      Dummy({'type': m.type})]}
       get_call_content = take_addr_if_nonpointer(m) + 'self->' + attr + '[arg0]'
       set_call_content = 'self->' + attr + '[arg0] = ' + deref_if_nonpointer(m) + 'arg1'
       if m.getExtendedAttribute('BoundsChecked'):
@@ -668,8 +713,8 @@ for name in names:
         get_call_content = "(%s, %s)" % (bounds_check, get_call_content)
         set_call_content = "(%s, %s)" % (bounds_check, set_call_content)
     else:
-      get_sigs = { 0: [] }
-      set_sigs = { 1: [Dummy({ 'type': m.type })] }
+      get_sigs = {0: []}
+      set_sigs = {1: [Dummy({'type': m.type})]}
       get_call_content = take_addr_if_nonpointer(m) + 'self->' + attr
       set_call_content = 'self->' + attr + ' = ' + deref_if_nonpointer(m) + 'arg0'
 
@@ -707,12 +752,11 @@ for name in names:
       mid_js += [r'''
     Object.defineProperty(%s.prototype, '%s', { get: %s.prototype.%s, set: %s.prototype.%s });''' % (name, attr, name, get_name, name, set_name)]
 
-
   if not interface.getExtendedAttribute('NoDelete'):
     mid_js += [r'''
   %s.prototype['__destroy__'] = %s.prototype.__destroy__ = ''' % (name, name)]
     render_function(name,
-                    '__destroy__', { 0: [] }, 'Void',
+                    '__destroy__', {0: []}, 'Void',
                     None,
                     None,
                     None,
@@ -750,8 +794,7 @@ for name, enum in enums.items():
       [namespace, identifier] = symbols
       if namespace in interfaces:
         # namespace is a class
-        deferred_js += ["Module['%s']['%s'] = _%s();\n" % \
-                  (namespace, identifier, function_id)]
+        deferred_js += ["Module['%s']['%s'] = _%s();\n" % (namespace, identifier, function_id)]
       else:
         # namespace is a namespace, so the enums get collapsed into the top level namespace.
         deferred_js += ["Module['%s'] = _%s();\n" % (identifier, function_id)]
@@ -759,22 +802,25 @@ for name, enum in enums.items():
       raise Exception("Illegal enum value %s" % value)
 
 mid_c += ['\n}\n\n']
-mid_js += ['''
+if len(deferred_js):
+  mid_js += ['''
 (function() {
   function setupEnums() {
     %s
   }
   if (runtimeInitialized) setupEnums();
-  else addOnPreMain(setupEnums);
+  else addOnInit(setupEnums);
 })();
 ''' % '\n    '.join(deferred_js)]
 
 # Write
 
 with open(output_base + '.cpp', 'w') as c:
-  for x in pre_c: c.write(x)
-  for x in mid_c: c.write(x)
+  for x in pre_c:
+    c.write(x)
+  for x in mid_c:
+    c.write(x)
 
 with open(output_base + '.js', 'w') as js:
-  for x in mid_js: js.write(x)
-
+  for x in mid_js:
+    js.write(x)
